SystemOrganization addCategory: #'Collections-Graphs'!


Object subclass: #DAGFrontier
	instanceVariableNames: 'frontier bag '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

Object subclass: #Dijkstra
	instanceVariableNames: 'graph source predecessor distance '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

Collection subclass: #Graph
	instanceVariableNames: 'nodes nodeCreator type '
	classVariableNames: 'InitializationBlocks '
	poolDictionaries: ''
	category: 'Collections-Graphs'!

PackageInfo subclass: #GraphInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

Object subclass: #GraphNode
	instanceVariableNames: 'value '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

GraphNode subclass: #ExplicitGraphNode
	instanceVariableNames: 'neighbors '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

GraphNode subclass: #ImplicitGraphNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

ImplicitGraphNode subclass: #ImplicitGraphNodeCollection
	instanceVariableNames: 'collectionBlock '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

ImplicitGraphNode subclass: #ImplicitGraphNodeIterator
	instanceVariableNames: 'iterator '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

ExplicitGraphNode subclass: #LabelledExplicitGraphNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

Graph subclass: #RootedGraph
	instanceVariableNames: 'roots '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Graphs'!

DAGFrontier class
	instanceVariableNames: ''!

Dijkstra class
	instanceVariableNames: ''!

Graph class
	instanceVariableNames: ''!

GraphInfo class
	instanceVariableNames: ''!

GraphNode class
	instanceVariableNames: ''!

ExplicitGraphNode class
	instanceVariableNames: ''!

ImplicitGraphNode class
	instanceVariableNames: ''!

ImplicitGraphNodeCollection class
	instanceVariableNames: ''!

ImplicitGraphNodeIterator class
	instanceVariableNames: ''!

LabelledExplicitGraphNode class
	instanceVariableNames: ''!

RootedGraph class
	instanceVariableNames: ''!


!DAGFrontier commentStamp: '<historical>' prior: 0!
A DAGFrontier is used to enumerate a directed acyclic graph (DAG) respecting the dependencies between nodes.  A client does not create it directly, but sends the message "frontier" to the DAG in question.

At any moment it holds a "frontier" of nodes that can be accessed.  When any node in the frontier has been processed, it can be removed from the frontier, and the frontier will be updated if necessary (in situ).

Instance variables:
frontier		<Set> of nodes on the frontier, empty if the whole DAG has been enumerated.
bag		<Bag> of nodes, used to calculate updates to the frontier.

Clients of this class must *not* modify the frontier set, only access it.
!

!ExplicitGraphNode commentStamp: '<historical>' prior: 0!
An ExplicitGraphNode stores the edges in a collection, an instance variable of self.
The type of the collection varies as to whether the graph is ordered, of fixed arity, etc.

To change the edge collection, addNeighbor: and removeNeighbor: messages are supplied.
!

!Graph commentStamp: '<historical>' prior: 0!
A Graph is a collection of GraphNodes.

Various properties of the graph are encoded in the class of its nodes (eg whether the graph is ordered, has a fixed arity, or the edges or labelled).
The creation block is initialized so that appropriate nodes are made.

Instance variable:
nodes:	a KeyedSet of GraphNodes
nodeCreator: a block which is evaluated to create a new node.
type:	a Symbol indicating what the default node type is.

PS: It's still not clear to me whether there's much to be gained by having this under Collection.  Little is inherited; much of what is inherited has no obvious interpretation or meaning, so why bother?  Problems, problems...!

!GraphNode commentStamp: '<historical>' prior: 0!
GraphNode is an abstract class of nodes that are held in a graph.

Each node holds on to a corresponding object that is the value of that node.

Subclasses add state/behaviour to represent edges in the graph.!

!ImplicitGraphNode commentStamp: '<historical>' prior: 0!
An ImplicitGraphNode has the edge structure represented within the value, and therefore forwards messages to the value to access edges.

To change the edge collection, you must send messages to the value.

Subclasses decide as to whether the value can yield a collection of edges more efficiently than iterating over them.!

!ImplicitGraphNodeCollection commentStamp: '<historical>' prior: 0!
I access the collection of edges by evaluating my block.!

!ImplicitGraphNodeIterator commentStamp: '<historical>' prior: 0!
I can iterate over the collection of edges by evaluating the iterator.!

!LabelledExplicitGraphNode commentStamp: '<historical>' prior: 0!
A LabelledExplicitGraphNode associates a label with each neighbor.

When adding neighbors, the label must be given.!

!RootedGraph commentStamp: '<historical>' prior: 0!
A RootedGraph is a directed graph with roots.

Instance variable:
roots:	Set of root objects!

!GraphNode methodsFor: 'comparing'!
= anObject
	"Comparison is delegated to the value of the node.
	Argument and receiver are reversed to dereference through other GraphNodes."
	^anObject = value! !

!Graph methodsFor: 'adding'!
add: aGraphNode 
	^nodes add: aGraphNode! !

!Graph methodsFor: 'adding'!
addEdge: edge 
	"Add an edge (represented by an Association)"
	self addEdgeFrom: edge key to: edge value.
	self triggerEvent: #addEdge: with: edge.
	type = #(#undirected )
		ifTrue: [| reversedEdge | 
			reversedEdge _ edge value -> edge key.
			self addEdgeFrom: edge value to: edge key.
			self triggerEvent: #addEdge: with: reversedEdge].
	^ edge!
]style[(9 4 3 45 2 4 14 4 9 4 9 4 15 9 7 4 3 4 3 15 14 12 7 12 3 4 10 4 9 4 14 4 11 4 9 4 15 9 7 12 6 4)f1b,f1cblue;b,f1,f1c152050000,f1,f1cmagenta;,f1,f1cblue;i,f1,f1cblue;i,f1,f1cmagenta;,f1,f1c202202126,f1,f1cblue;i,f1,f1cmagenta;,f1,f1c202202126,f1,f1cblue;i,f1,f1cblue;i,f1,f1cblue;i,f1,f1cblue;i,f1,f1cmagenta;,f1,f1cblue;i,f1,f1cblue;i,f1,f1cmagenta;,f1,f1c202202126,f1,f1cblue;i,f1,f1cblue;i! !

!Graph methodsFor: 'adding'!
addEdge: edge label: label
	"Add an edge (represented by an Association), with given label.
	pre: (self nodeFor: edge key) isLabelled"

	self addEdgeFrom: edge key to: edge value label: label.
	^edge! !

!Graph methodsFor: 'adding'!
addEdgeFrom: source to: target 
	| src trgt |
	src _ self addNode: source.
	trgt _ self addNode: target.
	src addNeighbor: trgt!
]style[(13 6 5 6 4 9 4 3 3 4 10 6 3 4 3 4 10 6 3 3 14 4)f1b,f1cblue;b,f1b,f1cblue;b,f1,f1cblue;i,f1,f1cblue;i,f1,f1cmagenta;,f1,f1cblue;i,f1,f1cblue;i,f1,f1cmagenta;,f1,f1cblue;i,f1,f1cblue;i,f1,f1cblue;i! !

!RootedGraph methodsFor: 'adding'!
addEdgeFrom: source to: target
	super addEdgeFrom: source to: target.
	(roots includes: target) ifTrue: [roots remove: target ifAbsent: []]! !

!Graph methodsFor: 'adding'!
addEdgeFrom: source to: target label: label
	"pre: (self nodeFor: edge key) isLabelled"
	| src trgt |
	src := self addNode: source.
	trgt := self addNode: target.
	src addNeighbor: trgt label: label! !

!RootedGraph methodsFor: 'adding'!
addEdgeFrom: source to: target label: label
	super addEdgeFrom: source to: target label: label.
	(roots includes: target) ifTrue: [roots remove: target ifAbsent: []]! !

!ExplicitGraphNode methodsFor: 'changing edges'!
addNeighbor: aNode 
	(neighbors includes: aNode)
		ifFalse: [neighbors add: aNode.
			self triggerEvent: #addNeighbor: with: aNode].
	^ aNode! !

!LabelledExplicitGraphNode methodsFor: 'changing edges'!
addNeighbor: aNode
	self shouldNotImplement! !

!ExplicitGraphNode methodsFor: 'changing edges' stamp: 'len 2/12/2003 21:41'!
addNeighbor: aNode at: index
	"pre: self isOrdered"
	^ neighbors at: index put: aNode! !

!LabelledExplicitGraphNode methodsFor: 'changing edges'!
addNeighbor: aNode at: index
	self shouldNotImplement! !

!LabelledExplicitGraphNode methodsFor: 'changing edges'!
addNeighbor: aNode at: index label: label
	"pre: self isOrdered"
	^super addNeighbor: aNode -> label at: index! !

!LabelledExplicitGraphNode methodsFor: 'changing edges'!
addNeighbor: aNode label: label
	^super addNeighbor: aNode -> label! !

!Graph methodsFor: 'adding'!
addNode: aSymbolNode 
	"Create a new GraphNode for the value (if necessary), and add it to the  
	graph. You shouldn't be calling this unless you're sure what you're  
	doing. Better to add nodes by adding edges."
	^ nodes
		at: aSymbolNode
		ifAbsent: [self
				add: (self newNodeOn: aSymbolNode).
			self
				triggerEvent: #addNode:
				with: (self getNode: aSymbolNode).
				 nodes at: aSymbolNode.]! !

!RootedGraph methodsFor: 'adding'!
addRoot: root
	"Add root as a root of the graph."
	roots add: root.
	^nodes add: (self newNodeOn: root)! !

!ExplicitGraphNode class methodsFor: 'instance creation'!
arity: n
	"Create a new ordered node with fixed arity n."
	^super new initializeFixed: n! !

!Graph methodsFor: 'initialization'!
arity: n
	type := #(#arity).
	nodeCreator := (InitializationBlocks at: #arity) value: n! !

!Graph class methodsFor: 'instance creation'!
arity: n
	"Create a new ordered graph with fixed arity."
	^super new initialize arity: n! !

!Graph methodsFor: 'initialization'!
arityLabelled: n
	type := #(#arity #labelled).
	nodeCreator := (InitializationBlocks at: #arityLabelled) value: n! !

!Graph class methodsFor: 'instance creation'!
arityLabelled: n
	"Create a new labelled, ordered graph with fixed arity."
	^super new initialize arityLabelled: n! !

!ExplicitGraphNode class methodsFor: 'instance creation'!
binary
	"Create a new binary node."
	^super new initializeFixed: 2! !

!Graph methodsFor: 'initialization'!
binary
	type := #(#binary).
	nodeCreator := InitializationBlocks at: #binary! !

!Graph class methodsFor: 'instance creation'!
binary
	"Create an ordered graph with arity of 2 (ie each vertex has exactly two connecting edges)."
	^super new initialize binary! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: 'len 2/12/2003 21:40'!
binary: value
	"Create a new binary node, and given value."
	^super new initializeFixed: 2; value: value! !

!Graph class methodsFor: 'instance creation'!
binaryLabelled
	"Create an labelled, ordered graph with arity of 2 (ie each vertex has exactly two connecting edges)."
	^super new initialize binaryLabelled! !

!Graph methodsFor: 'initialization'!
binaryLabelled
	type := #(#binary #labelled).
	nodeCreator := InitializationBlocks at: #binaryLabelled! !

!RootedGraph methodsFor: 'enumerating'!
breadthFirstDo: aBlock
	"Evaluate aBlock for each set of nodes at a given depth in the graph.
	A node is at depth n if the longest path to that node from a root has n steps.
	pre: self isCyclic not"

	| df current |
	df := self frontier.
	[current := df frontier copy.
	current isEmpty] whileFalse:
		[aBlock value: current.
		df removeAll: current]! !

!RootedGraph methodsFor: 'adding'!
buildFromRoots: rootCollection
	"Build the graph with given roots.
	The nodes should all be ImplicitGraphNodes, or have outgoing edges attached."
	self roots: rootCollection.
	self markDo: [ :junk ]! !

!ImplicitGraphNodeCollection methodsFor: 'initialisation'!
collectionBlock: aBlock
	collectionBlock := aBlock! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/14/2003 22:56'!
components
	"Answer the Strongly Connected Components of the receiver."
	^ Set accumulate: [:aBlock | self componentsDo: aBlock]! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/14/2003 22:56'!
componentsDo: aBlock
	"Enumerate the Strongly Connected Components of the receiver."
	| component remaining node |
	remaining _ self asSet.
	[remaining isEmpty]
		whileFalse: [node _ remaining anyOne.
					component _ self copyEmpty.
					node markDo: [:each |
						component add: each.
						remaining remove: each ifAbsent: []].
					aBlock value: component]! !

!ExplicitGraphNode methodsFor: 'copying' stamp: 'len 2/12/2003 21:43'!
copy
	^ super copy setNeighbors: neighbors copy! !

!Graph methodsFor: 'copying'!
copyEmpty
	"Return a graph of the same type with no nodes or edges."
	^self isImplicit
		ifTrue: [self class ordered]
		ifFalse: [self copy initialize]! !

!RootedGraph methodsFor: 'copying'!
copyEmpty
	"Return a new graph of the same type, with the same roots, but no edges."
	| copy |
	copy := super copyEmpty.
	copy roots: (roots copyEmpty: roots size).
	^copy! !

!Graph methodsFor: 'copying'!
copyNodes: nodeBlock
	"Make a copy with the same number of nodes and edges.
	Each node is self is mapped though nodeBlock"
	^self copyNodes: nodeBlock labels: nil! !

!Graph methodsFor: 'copying'!
copyNodes: nodeBlock  labels: labelBlock
	"Make a copy with the same number of nodes and edges.
	Each node is self is mapped though nodeBlock, labels (if used) are
	mapped through labelBlock (if non-nil)."
	| newGraph map |
	newGraph := self copyEmpty.
	map := DictionaryWithDefault newWithDefaultValueBlock: nodeBlock.
	self do: [ :node |
		node neighborsAndLabelsDo: [ :n :label |
			(labelBlock notNil and: [label notNil])
				ifTrue: [newGraph 
							addEdgeFrom: (map at: node value)
							to: (map at: n value)
							label: (labelBlock value: label)]
				ifFalse: [newGraph
							addEdgeFrom: (map at: node value)
							to: (map at: n value)]]].
	^newGraph! !

!RootedGraph methodsFor: 'copying'!
copyNodes: nodeBlock  labels: labelBlock
	"Make a copy with the same number of nodes and edges.
	Each node is self is mapped though nodeBlock, labels (if used) are
	mapped through labelBlock (if non-nil)."
	| newGraph map |
	newGraph := self copyEmpty.
	map := DictionaryWithDefault newWithDefaultValueBlock: nodeBlock.
	newGraph roots: (roots collect: [ :root | map at: root]).
	self do: [ :node |
		node neighborsAndLabelsDo: [ :n :label |
			(labelBlock notNil and: [label notNil])
				ifTrue: [newGraph 
							addEdgeFrom: (map at: node value)
							to: (map at: n value)
							label: (labelBlock value: label)]
				ifFalse: [newGraph
							addEdgeFrom: (map at: node value)
							to: (map at: n value)]]].
	^newGraph! !

!ExplicitGraphNode methodsFor: 'accessing edges'!
degree
	^neighbors size! !

!GraphNode methodsFor: 'accessing edges'!
degree
	"How many edges does this node have?"
	self subclassResponsibility! !

!ImplicitGraphNodeCollection methodsFor: 'accessing edges'!
degree
	^self neighbors size! !

!ImplicitGraphNodeIterator methodsFor: 'accessing edges'!
degree
	^iterator size! !

!RootedGraph methodsFor: 'graph operations'!
depthList
	"Partitions the DAG, returning a SequenceableCollection of Sets of nodes in the graph.  Element n in the SequenceableCollection is the Set of nodes in the graph reachable in n-1 steps by the longest route from a root."
	^OrderedCollection accumulate: [ :incBlock | self breadthFirstDo: incBlock]! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/13/2003 16:56'!
distanceFrom: source to: target
	^ (self shortestPathFrom: source to: target) size! !

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/13/2003 21:56'!
distanceTo: aGraphNode
	^ distance at: aGraphNode! !

!Graph methodsFor: 'enumerating'!
do: aBlock
	self nodesDo: aBlock! !

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/13/2003 21:56'!
eccentricity
	"Answer the eccentricity of the source vertex.
	The eccentricity of a vertex is the length of the longest minimal path from that vertex to some vertex in the graph. You can think of the eccentricity of a vertex as the longest distance in the graph from there to somewhere."

	^ distance max! !

!Graph methodsFor: 'accessing'!
edges
	^OrderedCollection accumulate: [ :incBlock | self edgesDo: incBlock]! !

!Graph methodsFor: 'enumerating'!
edgesDo: aBlock
	self nodesDo: [ :node |
		node neighborsDo: [ :n |
			aBlock value: node -> n]]! !

!RootedGraph class methodsFor: 'examples'!
exampleDAG
	"RootedGraph exampleDAG."
	| d |
	d := self unordered.
	d addRoot: #r1.
	d addRoot: #r2.
	d addEdge: #r1 -> #n1.
	d addEdge: #r1 -> #n2.
	d addEdge: #r2 -> #n2.
	d addEdge: #n1 -> #n3.
	d addEdge: #n2 -> #n3.
	^d! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 22:59'!
exampleForest
	"RootedGraph exampleForest"
	^self unordered roots: (Set with: Number with: Stream) children: [ :class | class subclasses]! !

!Graph class methodsFor: 'examples'!
exampleGraph
	"Graph exampleGraph."
	| d |
	d _ self ordered.
	d addEdge: #r1 -> #n1.
	d addEdge: #r1 -> #n2.
	d addEdge: #r2 -> #n2.
	d addEdge: #n1 -> #n3.
	d addEdge: #n2 -> #n3.
	d addEdge: #n3 -> #r1.
	^ d! !

!Graph class methodsFor: 'examples'!
exampleGraph2
	"Graph exampleGraph2."
	| d |
	d _ self unordered.
d addEdge: #a->#b.
	^ d! !

!Graph class methodsFor: 'examples' stamp: 'Samir Saidani 9/17/2003 18:49'!
exampleGraph3
	"Graph exampleGraph."
	| d |
	d _ self undirected.
	d addEdge: #r1 -> #n1.
	d addEdge: #r1 -> #n2.
	d addEdge: #r2 -> #n2.
	d addEdge: #n1 -> #n3.
	d addEdge: #n2 -> #n3.
	d addEdge: #n3 -> #r1.
	^ d! !

!Graph class methodsFor: 'examples'!
exampleImplicitGraph
	"Graph exampleImplicitGraph."
	| tree |
	tree := self implicitCollection: [ :class | class subclasses].
	Number withAllSubclasses do: [ :class | tree addNode: class].
	^tree! !

!Graph class methodsFor: 'examples'!
exampleImplicitGraph2
	"Graph exampleImplicitGraph2"
	| tree |
	tree := self implicitIteratorBlock: [ :class | [ :aBlock | class subclasses do: aBlock]].
	Number withAllSubclasses do: [ :class | tree addNode: class].
	^tree! !

!RootedGraph class methodsFor: 'examples'!
exampleImplicitLargeTree
	"RootedGraph exampleImplicitLargeTree"
	^(self
		implicitCollection:
			[ :class |
				class == Class
					ifTrue: [#()]
					ifFalse: [class subclasses asSortedCollection: [ :x :y | x name < y name]]]) 		buildFromRoots: (Array with: Object)! !

!RootedGraph class methodsFor: 'examples'!
exampleImplicitSmallTree
	"RootedGraph exampleImplicitSmallTree"
	^(self implicitCollection: [ :class | class subclasses]) buildFromRoots: (Set with: Number)! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 22:58'!
exampleLargeTree
	"RootedGraph exampleLargeTree"
	^self ordered roots: (Array with: Object) children:
		[ :class | class == Class
					ifTrue: [#()]
					ifFalse: [class subclasses asSortedCollection: [ :x :y | x name < y name]]]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 22:59'!
exampleMediumLabelledTree
	"RootedGraph exampleMediumLabelledTree"
	^self unorderedLabelled
		roots: (Set with: Collection)
		children: [ :class | class subclasses]
		label: [ :src :trg |
				trg isVariable
					ifTrue: [trg isBits ifTrue: [#variableByte] ifFalse: [#variable]]
					ifFalse: [#normal]]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/14/2003 22:58'!
exampleMediumTree
	^self unordered roots: (Set with: Stream) children: [ :class | class subclasses]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/14/2003 23:43'!
exampleMediumTree2
	^self unordered roots: (Set with: Collection) children: [ :class | class subclasses]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 23:00'!
exampleSmallLabelledTree
	"RootedGraph exampleSmallLabelledTree"
	^self unorderedLabelled
		roots: (Set with: Magnitude)
		children: [ :class | class subclasses]
		label: [ :src :trg | trg category]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 23:00'!
exampleSmallLabelledTree2
	"RootedGraph exampleSmallLabelledTree2"
	^self unorderedLabelled
		roots: (Set with: Magnitude)
		children: [ :class | class subclasses]
		label: [ :src :trg |
				trg isVariable
					ifTrue: [trg isBits ifTrue: [#variableByte] ifFalse: [#variable]]
					ifFalse: [#normal]]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/12/2003 22:42'!
exampleSmallTree
	"RootedGraph exampleSmallTree"
	^self unordered roots: (Set with: Number) children: [ :class | class subclasses]! !

!RootedGraph class methodsFor: 'examples' stamp: 'len 2/18/2003 22:09'!
exampleSmallTree2
	"RootedGraph exampleSmallTree2"
	^self unordered roots: (Set with: Magnitude) childrenMsg: #subclasses! !

!Graph class methodsFor: 'examples' stamp: 'Samir Saidani 6/19/2003 20:24'!
exampleUndirectedGraph
	"Graph exampleUndirectedGraph."
	| d |
	d _ self undirected.
	d addEdge: #a -> #b.
	^ d! !

!RootedGraph methodsFor: 'initialization' stamp: 'len 2/12/2003 22:29'!
findRoots
	"Find all nodes that are not the targets of an edge and make them the roots."
	roots := Set new addAll: nodes; yourself.
	self nodesDo: [:node |
		node neighborsDo: [ :nb |
			roots remove: nb ifAbsent: []]]! !

!DAGFrontier methodsFor: 'accessing'!
frontier
	^frontier! !

!RootedGraph methodsFor: 'enumerating'!
frontier
	"Return a DAGFrontier object, capable of enumerating this DAG."
	^DAGFrontier frontierOn: self! !

!DAGFrontier methodsFor: 'initialization'!
frontier: f bag: remaining
	frontier := f.
	bag := remaining! !

!DAGFrontier class methodsFor: 'instance creation'!
frontierOn: rootedGraph
	"Create a frontier object to enumerate the graph."
	| remaining frontier |
	"Build a Bag of non-root nodes, each node once in the Bag for every predecessor."
	frontier := rootedGraph rootNodes asOrderedCollection.
	remaining := Bag new.
	rootedGraph nodesDo: [ :node | node neighborsDo: [ :n | remaining add: n]].
	^self new frontier: frontier bag: remaining! !

!Graph methodsFor: 'accessing'!
getNode: aSymbol 
	^ self nodes at: aSymbol!
]style[(9 7 5 4 11 7)f1b,f1cblue;b,f1,f1cmagenta;,f1,f1cblue;i! !

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/15/2003 18:52'!
graph: aGraph source: aNode
	graph _ aGraph.
	source _ aNode.
	self run! !

!ExplicitGraphNode methodsFor: 'testing edges'!
hasEdgeTo: anObject
	^neighbors includes: anObject! !

!GraphNode methodsFor: 'testing edges'!
hasEdgeTo: anObject
	"Is there an edge from this node to anObject?"
	self subclassResponsibility! !

!ImplicitGraphNodeCollection methodsFor: 'testing edges'!
hasEdgeTo: anObject
	^self neighbors includes: anObject! !

!ImplicitGraphNodeIterator methodsFor: 'testing edges'!
hasEdgeTo: anObject
	^iterator includes: anObject! !

!LabelledExplicitGraphNode methodsFor: 'testing edges'!
hasEdgeTo: anObject
	^neighbors anySatisfy: [ :assoc | assoc key = anObject]! !

!GraphNode methodsFor: 'comparing'!
hash
	^value hash! !

!Graph class methodsFor: 'instance creation'!
implicitCollection: collectionBlock
	"Create a new graph, using the structure implicit in existing objects.
	Each node is the graph is accessed by evaluating collectionBlock to yield a collection of neighbouring nodes."
	^super new initialize implicitCollection: collectionBlock! !

!Graph methodsFor: 'initialization'!
implicitCollection: collectionBlock
	type := #(#implicitCollection).
	nodeCreator := (InitializationBlocks at: #implicitCollection) value: self value: collectionBlock! !

!Graph methodsFor: 'initialization'!
implicitIteratorBlock: iteratorBlock
	type := #(#implicitIteratorBlock).
	nodeCreator := (InitializationBlocks at: #implicitIteratorBlock) value: self value: iteratorBlock! !

!Graph class methodsFor: 'instance creation'!
implicitIteratorBlock: iteratorBlock
	"Create a new graph, using the structure implicit in existing objects.
	Each node is the graph is accessed by evaluating iteratorBlock to iterate over a collection of neighbouring nodes."
	^super new initialize implicitIteratorBlock: iteratorBlock! !

!Graph methodsFor: 'testing'!
includes: anObject 
	"Answer whether anObject is one of the receiver's elements."

	^nodes includes: anObject! !

!Graph methodsFor: 'initialization'!
initialize
	nodes := KeyedSet new! !

!Graph class methodsFor: 'initialization'!
initialize
	"Graph initialize"
	| d |
	d _ Dictionary new.
	d
		at: #binary
		put: [:value | ExplicitGraphNode binary: value].
	d
		at: #arity
		put: [:n | [:value | ExplicitGraphNode on: value arity: n]].
	d
		at: #binaryLabelled
		put: [:value | LabelledExplicitGraphNode binary: value].
	d
		at: #ordered
		put: [:value | ExplicitGraphNode ordered: value].
	d
		at: #unordered
		put: [:value | ExplicitGraphNode newOn: value].
d
		at: #undirected
		put: [:value | ExplicitGraphNode newOn: value].
	d
		at: #arityLabelled
		put: [:n | [:value | LabelledExplicitGraphNode on: value arity: n]].
	d
		at: #orderedLabelled
		put: [:value | LabelledExplicitGraphNode ordered: value].
	d
		at: #unorderedLabelled
		put: [:value | LabelledExplicitGraphNode newOn: value].
	d
		at: #implicitCollection
		put: [:graph :collectionBlock | [:value | ImplicitGraphNode
				on: value
				collectionBlock: [(collectionBlock value: value)
						collect: [:node | graph addNode: node]]]].
	d
		at: #implicitIteratorBlock
		put: [:graph :iteratorBlock | [:value | 
			| iterator | 
			iterator _ iteratorBlock value: value.
			ImplicitGraphNode
				on: value
				iteratorBlock: [:aBlock | iterator
						value: [:node | aBlock
								value: (graph addNode: node)]]]].
	InitializationBlocks _ d! !

!RootedGraph methodsFor: 'initialization'!
initialize
	super initialize.
	roots := Set new! !

!ExplicitGraphNode methodsFor: 'initialize'!
initializeFixed: arity
	neighbors := Array new: arity! !

!ExplicitGraphNode methodsFor: 'initialize'!
initializeOrdered
	neighbors := OrderedCollection new! !

!ExplicitGraphNode methodsFor: 'initialize'!
initializeUnordered
	neighbors := Set new! !

!RootedGraph methodsFor: 'testing' stamp: 'len 2/15/2003 23:06'!
isCyclic
	"Is the graph cyclic?"
	| frontier remaining |
	frontier := self rootNodes.
	"Build a Bag of non-root nodes, each node once in the Bag for every predecessor."
	remaining := Bag new.
	nodes do: [ :node | remaining addAll: node neighbors].
	[| next |
	next := frontier anyIfNone: [^remaining isEmpty not].
	frontier remove: next.
	next neighborsDo:
		[ :neighbor |
			(remaining removeOne: neighbor) = 0
				ifTrue: [frontier add: neighbor]]
	] repeat! !

!Graph methodsFor: 'testing'!
isEmpty
	"Answer whether the receiver contains any elements."

	^nodes isEmpty! !

!RootedGraph methodsFor: 'testing'!
isEmpty
	"Answer whether the receiver contains any elements."

	^roots isEmpty! !

!Graph methodsFor: 'private'!
isImplicit
	"Is the node creator for implicit nodes?"
	^(type includes: #implicitCollection) or: [type includes: #implicitIteratorBlock]! !

!ExplicitGraphNode methodsFor: 'testing'!
isLabelled
	"Are the edges from this node labelled?"
	^false! !

!Graph methodsFor: 'private'!
isLabelled
	"Is the node creator for labelled nodes?"
	^type includes: #labelled! !

!GraphNode methodsFor: 'testing'!
isLabelled
	"Are the edges from this node labelled?"
	self subclassResponsibility! !

!LabelledExplicitGraphNode methodsFor: 'testing'!
isLabelled
	"Are the edges from this node labelled?"
	^true! !

!GraphNode methodsFor: 'testing edges'!
isLeaf
	^self degree = 0! !

!ExplicitGraphNode methodsFor: 'testing'!
isOrdered
	^neighbors isSequenceable! !

!RootedGraph methodsFor: 'testing'!
isTree
	"Is the graph a tree (more accurately, a forest)?
	It is if every node has at most one predecessor."
	| visited |
	visited := Set new.
	nodes do: [ :node |
		node neighborsDo: [ :neighbor |
			(visited includes: neighbor) ifTrue: [^false].
			visited add: neighbor]].
	^true! !

!ImplicitGraphNodeIterator methodsFor: 'initialisation'!
iterator: anIterator
	iterator := anIterator! !

!LabelledExplicitGraphNode methodsFor: 'accessing edges'!
labelOfEdgeTo: anObject
	"Return the label associated with the edge to anObject; otherwise signal an error."
	^(neighbors detect: [ :assoc | assoc key = anObject]) value! !

!LabelledExplicitGraphNode methodsFor: 'accessing edges'!
labelOfEdgeTo: anObject ifNone: aBlock
	"Return the label associated with the edge to anObject; return the value of the block is no such edge exists."
	^(neighbors detect: [ :assoc | assoc key = anObject] ifNone: [^aBlock value]) value! !

!GraphNode methodsFor: 'enumerating' stamp: 'len 2/18/2003 21:37'!
markDo: aBlock
	"Visit each node in the graph once, applying aBlock.
	A node is only visited after at least one of its predecessors, but not necessarily after all the predecessors."
	| todo visited |
	todo _ Set with: self.
	visited := Set new.
	[todo isEmpty] whileFalse:
		[| node |
		node := todo anyOne.
		visited add: node.
		aBlock value: node.
		node neighborsDo:
			[ :child |
				(visited includes: child)
					ifFalse: [todo add: child]].
		todo remove: node]! !

!RootedGraph methodsFor: 'enumerating' stamp: 'len 2/18/2003 21:37'!
markDo: aBlock
	"Visit each node in the graph once, applying aBlock.
	A node is only visited after at least one of its predecessors, but not necessarily after all the predecessors."
	| todo visited |
	todo := self rootNodes asSet.
	visited := Set new.
	[todo isEmpty] whileFalse:
		[| node |
		node := todo anyOne.
		visited add: node.
		aBlock value: node.
		node neighborsDo:
			[ :child |
				(visited includes: child)
					ifFalse: [todo add: child]].
		todo remove: node]! !

!ExplicitGraphNode methodsFor: 'accessing edges'!
neighbors
	^neighbors! !

!GraphNode methodsFor: 'accessing edges'!
neighbors
	"Return a collection of nodes connected to outgoing edges."
	self subclassResponsibility! !

!ImplicitGraphNode methodsFor: 'accessing edges'!
neighbors
	^OrderedCollection accumulate: [ :incBlock | self neighborsDo: incBlock]! !

!ImplicitGraphNodeCollection methodsFor: 'accessing edges'!
neighbors
	^collectionBlock value! !

!LabelledExplicitGraphNode methodsFor: 'accessing edges'!
neighbors
	^neighbors collect: [ :assoc | assoc key]! !

!GraphNode methodsFor: 'accessing edges'!
neighborsAndLabelsDo: aBlock
	"Evaluate aBlock for each node connected to an outgoing edge, and the label on that edge (nil if no label)."
	"The default definition assumes no labels are present."
	self neighborsDo: [ :n | aBlock value: n value: nil]! !

!LabelledExplicitGraphNode methodsFor: 'accessing edges'!
neighborsAndLabelsDo: aBlock
	"Evaluate aBlock for each node connected to an outgoing edge, and the label on that edge (nil if no label)."
	^neighbors do: [ :assoc | aBlock value: assoc key value: assoc value]! !

!ExplicitGraphNode methodsFor: 'accessing edges'!
neighborsDo: aBlock
	neighbors do: aBlock! !

!GraphNode methodsFor: 'accessing edges'!
neighborsDo: aBlock
	"Evaluate aBlock for each node connected to an outgoing edge."
	self subclassResponsibility! !

!ImplicitGraphNode methodsFor: 'accessing edges'!
neighborsDo: aBlock
	self neighbors do: aBlock! !

!ImplicitGraphNodeIterator methodsFor: 'accessing edges'!
neighborsDo: aBlock
	iterator do: aBlock! !

!LabelledExplicitGraphNode methodsFor: 'accessing edges'!
neighborsDo: aBlock
	^neighbors do: [ :assoc | aBlock value: assoc key]! !

!ExplicitGraphNode class methodsFor: 'instance creation'!
new
	"Create a new unordered node."
	^super new initializeUnordered! !

!Graph class methodsFor: 'private'!
new
	"^ self error: 'Graphs should not be created with new.'"
^super new.! !

!Graph methodsFor: 'private'!
newNodeOn: object
	^nodeCreator value: object! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: 'len 2/12/2003 21:40'!
newOn: value
	"Create a new unordered node, and given value."
	^super new initializeUnordered; value: value! !

!Graph methodsFor: 'private'!
nodeCreator: n
	nodeCreator := n! !

!Graph methodsFor: 'private'!
nodeFor: object	
	^nodes at: object! !

!Graph methodsFor: 'accessing' stamp: 'Samir Saidani 5/14/2003 16:40'!
nodes
^nodes! !

!Graph methodsFor: 'enumerating'!
nodesDo: aBlock
	"No particular order."
	nodes do: aBlock! !

!Graph methodsFor: 'errors' stamp: 'len 2/13/2003 23:29'!
notFoundError
	^ self error: 'object is not in the collection'! !

!Graph methodsFor: 'accessing'!
numberOfEdges
	"This is commonly known as the size of the graph, but the size message returns the order (number of vertices), as this fits in better with Smalltalk usage."
	^(Iterator on: self msg: #edgesDo:) size! !

!Graph methodsFor: 'testing'!
occurrencesOf: anObject 
	"Answer how many of the receiver's elements are equal to anObject."

	^(self includes: anObject) ifTrue: [1] ifFalse: [0]! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: 'len 2/12/2003 21:40'!
on: value arity: n
	"Create a new ordered node with fixed arity n, and given value."
	^super new initializeFixed: n; value: value! !

!ImplicitGraphNode class methodsFor: 'instance creation'!
on: aValue collectionBlock: aBlock
	^ImplicitGraphNodeCollection on: aValue collectionBlock: aBlock! !

!ImplicitGraphNodeCollection class methodsFor: 'instance creation'!
on: aValue collectionBlock: aBlock
	^(self new) value: aValue ; collectionBlock: aBlock! !

!ImplicitGraphNode class methodsFor: 'instance creation'!
on: aValue iterator: anIterator
	^ImplicitGraphNodeIterator on: aValue iterator: anIterator! !

!ImplicitGraphNodeIterator class methodsFor: 'instance creation'!
on: aValue iterator: anIterator
	^(self new) value: aValue ; iterator: anIterator! !

!ImplicitGraphNode class methodsFor: 'instance creation'!
on: aValue iteratorBlock: iteratorBlock
	^ImplicitGraphNodeIterator on: aValue iteratorBlock: iteratorBlock! !

!ImplicitGraphNodeIterator class methodsFor: 'instance creation'!
on: aValue iteratorBlock: iteratorBlock
	^self on: aValue iterator: (Iterator on: iteratorBlock)! !

!Graph methodsFor: 'accessing'!
order
	^nodes size! !

!ExplicitGraphNode class methodsFor: 'instance creation'!
ordered
	"Create a new ordered node."
	^super new initializeOrdered! !

!Graph class methodsFor: 'instance creation'!
ordered
	"Create a new ordered graph."
	^ super new initialize ordered!
]style[(7 2 29 4 5 23)f1b,f1,f1c152050000,f1,f1cmagenta;,f1! !

!Graph methodsFor: 'initialization'!
ordered
	type := #(#ordered).
	nodeCreator := InitializationBlocks at: #ordered! !

!ExplicitGraphNode class methodsFor: 'instance creation' stamp: 'len 2/12/2003 21:40'!
ordered: value
	"Create a new ordered node, and given value."
	^super new initializeOrdered; value: value! !

!Graph class methodsFor: 'instance creation'!
orderedLabelled
	"Create a new labelled, ordered graph."
	^super new initialize orderedLabelled! !

!Graph methodsFor: 'initialization'!
orderedLabelled
	type := #(#ordered #labelled).
	nodeCreator := InitializationBlocks at: #orderedLabelled! !

!RootedGraph methodsFor: 'enumerating'!
preOrderDo: aBlock
	"Walk the graph, evaluating aBlock for each node.  Evaluate each node before its successors, but in no particular order when there is a choice.
	pre: self isCyclic not"
	| frontier remaining | 
	frontier := self rootNodes.
	"Build a Bag of non-root nodes, each node once in the Bag for every predecessor."
	remaining := Bag new.
	nodes do: [ :node | remaining addAll: node neighbors].
	[| next |
	next := frontier anyIfNone: [^self].
	aBlock value: next.
	frontier remove: next.
	next neighborsDo:
		[ :neighbor |
			(remaining removeOne: neighbor) = 0
				ifTrue: [frontier add: neighbor]]
	] repeat! !

!GraphInfo methodsFor: 'as yet unclassified' stamp: 'Samir Saidani 9/17/2003 18:08'!
prerequisiteChangeSets
	"These are the names of DVS packages that get loaded before the main  
	change set"
	^ #('Collections-Misc' 'Iterator' )! !

!GraphNode methodsFor: 'printing' stamp: 'len 2/14/2003 23:38'!
printOn: aStream
	aStream nextPut: $[; print: value; nextPut: $]! !

!RootedGraph methodsFor: 'graph operations'!
reduce
	"Remove all redundant edges to form the transitive reduction.

	Based on Algorithm 4 in 'An Algorithm for Finding a Minimum Equivalent Graph of a Digraph', by D. M. Moyles and G. L. Thompson, JACM 16(3), July 1969, pp455-60 (with obvious simplifications for acyclic graphs)."
	
	self rootNodes do: [ :root | self reduceStep: root with: OrderedCollection new]! !

!RootedGraph methodsFor: 'graph operations'!
reduceSlow
	"Remove all redundant edges to form the transitive reduction.

	Based on Algorithm 4 in 'An Algorithm for Finding a Minimum Equivalent Graph of a Digraph', by D. M. Moyles and G. L. Thompson, JACM 16(3), July 1969, pp455-60 (with obvious simplifications for acyclic graphs).
	
	This version sometimes builds huge sets!!"
	
	| seqSet |
	seqSet := Set accumulate: [ :incBlock |
				self rootNodes do: [ :root | incBlock value: (Array with: root)]].

	[seqSet isEmpty] whileFalse:
		[| newSeqSet |
"Transcript show: seqSet size printString ; cr."
		newSeqSet := Set new.
		seqSet do: [ :seq || tail |
			tail := seq last.
			tail neighborsDo: [ :neighbor |
				seq do: [ :node |
					(node ~= tail and: [node hasEdgeTo: neighbor])
						ifTrue: [
"Transcript show: node printString, ' -> ', neighbor printString ; cr."
								self removeEdgeFrom: node to: neighbor]].
				newSeqSet add: (seq copyWith: neighbor)]].
		seqSet := newSeqSet]! !

!RootedGraph methodsFor: 'graph operations'!
reduceStep: node with: sofar
	node degree = 0 ifTrue: [^self].
	node neighbors copy do: [ :succ |
		sofar do: [ :done |
			(done hasEdgeTo: succ)
				ifTrue: [self removeEdgeFrom: done to: succ]].
		sofar addLast: node.
		self reduceStep: succ with: sofar.
		sofar removeLast]! !

!DAGFrontier methodsFor: 'advancing'!
remove: frontierObject
	frontier remove: frontierObject.
	frontierObject neighborsDo:
		[ :neighbor |
			(bag removeOne: neighbor) = 0
				ifTrue: [frontier add: neighbor]].
	^frontierObject! !

!Graph methodsFor: 'removing'!
remove: aGraphNode 
	"Remove the given node.
	pre: the node must be in the graph, and not the target of an edge."
	^self remove: aGraphNode ifAbsent: [self notFoundError]! !

!Graph methodsFor: 'removing'!
remove: aGraphNode ifAbsent: exceptionBlock 
	"Remove the given node, evaluate exceptionBlock if it's not part of the graph.
	pre: the node shouldn't be the target of an edge."
	^nodes remove: aGraphNode ifAbsent: exceptionBlock! !

!DAGFrontier methodsFor: 'advancing'!
removeAll: collection
	collection do: [ :n | self remove: n]! !

!RootedGraph methodsFor: 'removing' stamp: 'len 2/14/2003 20:06'!
removeAllFrom: aNode
	"Remove the subtree below aNode (and associated edges), excluding aNode itself.
	Note that nodes are removed from the graph without being sent any messages."
	aNode
		walkPre: [ :junk ]
		post: [ :child |
			child == aNode
				ifTrue: [aNode neighbors copy do: [ :n | aNode removeNeighbor: n]]
				ifFalse: [self remove: child]]! !

!Graph methodsFor: 'removing'!
removeEdge: edge 
	"Remove the given edge.  
	pre: the edge must be in the graph."
	self
		removeEdge: edge
		ifAbsent: [self notFoundError].
	self triggerEvent: #removeEdge: with: edge.
	type = #(#undirected )
		ifTrue: [| reversedEdge | 
			reversedEdge _ edge value -> edge key.
			self removeEdge: reversedEdge ifAbsent: [self notFoundError].
			self triggerEvent: #removeEdge: with: reversedEdge]! !

!Graph methodsFor: 'removing'!
removeEdge: edge ifAbsent: exceptionBlock 
	"Remove the given edge, evaluate exceptionBlock if it doesn't exist.
	pre: both the source and target of the edge must be in the graph."
	^self
		removeEdgeFrom: edge key
		to: edge value
		ifAbsent: exceptionBlock! !

!Graph methodsFor: 'removing'!
removeEdgeFrom: src to: dest
	"Remove the given edge.
	pre: the edge must be in the graph."
	^self removeEdgeFrom: src to: dest ifAbsent: [self notFoundError]! !

!Graph methodsFor: 'removing'!
removeEdgeFrom: src to: dest ifAbsent: exceptionBlock
	"Remove the given edge.
	pre: both the source and target of the edge must be in the graph."
	^self removeEdgeFromNode: (self nodeFor: src) toNode: (self nodeFor: dest) ifAbsent: exceptionBlock! !

!Graph methodsFor: 'private'!
removeEdgeFromNode: srcNode toNode: destNode ifAbsent: exceptionBlock
	"Remove the given edge, evaluate exceptionBlock if it doesn't exist.
	pre: both the source and target of the edge must be in the graph."
	(srcNode hasEdgeTo: destNode)
		ifTrue: [srcNode removeNeighbor: destNode.
				^self]
		ifFalse: [^exceptionBlock value]! !

!ExplicitGraphNode methodsFor: 'changing edges' stamp: 'len 2/12/2003 21:41'!
removeNeighbor: aNode
	^ neighbors remove: aNode! !

!LabelledExplicitGraphNode methodsFor: 'changing edges'!
removeNeighbor: aNode
	neighbors remove: (neighbors detect: [ :assoc | assoc key = aNode])! !

!RootedGraph methodsFor: 'accessing' stamp: 'len 2/13/2003 22:47'!
rootNode
	"return the GraphNode for the root.
	pre: roots size = 1"
	^self nodeFor: roots anyOne! !

!RootedGraph methodsFor: 'accessing'!
rootNodes
	"Return the GraphNodes for the roots."
	^roots collect: [ :root | self nodeFor: root]! !

!RootedGraph methodsFor: 'initialization'!
roots: aCollection
	roots := aCollection.
	roots do: [ :root | self addNode: root]! !

!RootedGraph methodsFor: 'adding'!
roots: rootNodes children: aBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain a Collection of its children."
	^self roots: rootNodes childrenGenerator:
		[ :node | [ :iterationBlock | (aBlock value: node) do: iterationBlock]]! !

!RootedGraph methodsFor: 'adding'!
roots: rootNodes children: aBlock label: labelBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain a Collection of its children."
	^self
		roots: rootNodes 
		childrenGenerator: [ :node | [ :iterationBlock | (aBlock value: node) do: iterationBlock]]
		label: labelBlock! !

!RootedGraph methodsFor: 'adding'!
roots: rootNodes childrenGenerator: aBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain an iterator over a Collection of its children."
	^self roots: rootNodes childrenGenerator: aBlock label: nil! !

!RootedGraph methodsFor: 'adding' stamp: 'len 2/18/2003 21:38'!
roots: rootNodes childrenGenerator: aBlock label: labelBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain an iterator over a Collection of its children.
	Evaluate labelBlock (if not nil) for each node and child to obtain a label for the edge."
	| todo |
	self roots: rootNodes.
	todo := rootNodes asSet copy.
	[todo isEmpty] whileFalse:
		[| node |
		node := todo anyOne.
		(aBlock value: node) value:
			[ :child |
				(self includes: child) ifFalse: [todo add: child].
				labelBlock isNil
					ifTrue: [self addEdgeFrom: node to: child]
					ifFalse: [self addEdgeFrom: node to: child label: (labelBlock value: node value: child)]].
			todo remove: node].
	^self! !

!RootedGraph methodsFor: 'adding' stamp: 'len 2/18/2003 21:38'!
roots: rootNodes childrenLabelGenerator: aBlock
	"Build the graph with given rootNodes.
	Evaluate aBlock for each node to obtain an iterator over a Collection of its children and labels."
	| todo |
	self roots: rootNodes.
	todo := rootNodes asSet.
	[todo isEmpty] whileFalse:
		[| node |
		node := todo anyOne.
		(aBlock value: node) value:
			[ :child :label |
				(self includes: child) ifFalse: [todo add: child].
				self addEdgeFrom: node to: child label: label].
			todo remove: node].
	^self! !

!RootedGraph methodsFor: 'adding'!
roots: rootNodes childrenMsg: msg
	"Build the graph with given rootNodes.
	Send msg to each node to obtain a Collection of its children."
	^self roots: rootNodes childrenGenerator:
		[ :node | [ :iterationBlock | (node perform: msg) do: iterationBlock]]! !

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/13/2003 21:56'!
run
	| queue u estimate |
	graph do: [:each |
		distance at: each put: Float infinity.
		predecessor at: each put: nil].
	distance at: source put: 0.

	queue _ Heap sortBlock: [:a :b | (distance at: a) <= (distance at: b)].
	queue addAll: graph.
	[queue isEmpty]
		whileFalse: [u _ queue removeFirst.
					(graph nodeFor: u) neighborsAndLabelsDo: [:each :weight |
						estimate _ (distance at: u) + (weight ifNil: [1]).
						(distance at: each) > estimate
							ifTrue: [distance at: each put: estimate.
									predecessor at: each put: u]]]! !

!ExplicitGraphNode methodsFor: 'private'!
setNeighbors: aCollection
	neighbors := aCollection! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/15/2003 18:51'!
shortestPathFrom: source to: target
	^ Dijkstra new graph: self source: source; shortestPathTo: target! !

!Dijkstra methodsFor: 'as yet unclassified' stamp: 'len 2/13/2003 21:55'!
shortestPathTo: aGraphNode
	| answer node predecessorNode |
	answer _ OrderedCollection new.
	[node _ aGraphNode.
	answer add: node.
	predecessorNode _ predecessorNode at: node.
	source = predecessorNode]
		whileFalse: [node _ predecessorNode].
	^ answer! !

!Graph methodsFor: 'accessing'!
size
	"The order of the graph."
	^nodes size! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/13/2003 22:09'!
symmetric
	| answer |
	answer _ self copy.
	self edgesDo: [:each | answer addEdge: each value -> each key].
	^ answer! !

!Graph methodsFor: 'graph operations' stamp: 'len 2/13/2003 16:47'!
transposed
	| answer |
	answer _ self copyEmpty.
	self edgesDo: [:each | answer addEdge: each value -> each key].
	^ answer! !

!Graph methodsFor: 'accessing' stamp: 'Samir Saidani 6/19/2003 20:17'!
type
	^type! !

!Graph class methodsFor: 'instance creation' stamp: 'Samir Saidani 6/19/2003 17:30'!
undirected
	"Create an undirected graph"
	^ super new initialize undirected.! !

!Graph methodsFor: 'initialization'!
undirected
	type _ #(#undirected ).
	nodeCreator _ InitializationBlocks at: #undirected! !

!Graph class methodsFor: 'instance creation'!
unordered
	"Create a new unordered graph."
	^super new initialize unordered! !

!Graph methodsFor: 'initialization'!
unordered
	type := #(#unordered).
	nodeCreator := InitializationBlocks at: #unordered! !

!Graph class methodsFor: 'instance creation'!
unorderedLabelled
	"Create a new labelled, unordered graph."
	^super new initialize unorderedLabelled! !

!Graph methodsFor: 'initialization'!
unorderedLabelled
	type := #(#unordered #labelled).
	nodeCreator := InitializationBlocks at: #unorderedLabelled! !

!GraphNode methodsFor: 'accessing value'!
value
	^value! !

!GraphNode methodsFor: 'accessing value'!
value: val
	value := val! !

!Graph methodsFor: 'enumerating'!
valuesDo: aBlock
	"No particular order."
	nodes do: [ :node | aBlock value: node value]! !

!GraphNode methodsFor: 'enumerating' stamp: 'len 2/14/2003 20:05'!
walkPre: preBlock post: postBlock
	"Recursively walk the subtree rooted at me.  Apply preBlock to each node, then walk the subtree below node, then apply postBlock to the node."
	preBlock value: self.
	self neighborsDo: [ :child | child walkPre: preBlock post: postBlock].
	postBlock value: self! !

!RootedGraph methodsFor: 'enumerating' stamp: 'len 2/14/2003 20:06'!
walkPre: preBlock post: postBlock
	"Recursively walk the tree(s).  Apply preBlock to each node, then walk the subtree below node, then apply postBlock to the node."
	self rootNodes do: [ :root | root walkPre: preBlock post: postBlock]! !
Graph initialize!
